import mt940
import pytest


# def test_incorrect_transaction_data():
#     transactions = mt940.parser.Transactions()
#
#     with pytest.raises(RuntimeError):
#         mt940.parser.Transaction(
#             transactions=transactions,
#             data='foo',
#             details='bar',
#         )
